/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
/**
 * Conditional validators
 */
class ConditionalValidators {
    /**
     * Custom validation with condition
     * @param {?} state condition to set the validators
     * @param {?} validators validators array to set
     * @param {?=} controls controls to detect changes in their values
     * @return {?}
     */
    static custom(state, validators, controls) {
        return (/**
         * @param {?} control
         * @return {?}
         */
        (control) => {
            this.cleanChangeSubscriptions(control);
            this.pushChangeSubscriptions(control, this.toRealControls(control, controls));
            if (typeof state === 'function' && state.call(control.parent)) {
                for (const validator of validators) {
                    /** @type {?} */
                    const errors = validator(control);
                    if (errors !== null) {
                        return errors;
                    }
                }
            }
            return null;
        });
    }
    /**
     * Validation with condition equal to value
     * @param {?} conditions conditions like object (key is control name)
     * @param {?} validators validators array
     * @param {?=} controls controls to detect changes in their values
     * @return {?}
     */
    static equal(conditions, validators, controls) {
        return (/**
         * @param {?} control
         * @return {?}
         */
        (control) => {
            if (!(controls instanceof Array)) {
                controls = [];
            }
            /** @type {?} */
            let validate = true;
            for (const controlName in conditions) {
                if (!conditions.hasOwnProperty(controlName)) {
                    continue;
                }
                /** @type {?} */
                const value = conditions[controlName];
                /** @type {?} */
                const conditionalControl = this.getControlByName(controlName, (/** @type {?} */ (control.parent)));
                if (conditionalControl && controls.findIndex((/**
                 * @param {?} e
                 * @return {?}
                 */
                e => e === conditionalControl)) === -1) {
                    controls.push(conditionalControl);
                }
                if (conditionalControl !== null && conditionalControl.value !== value) {
                    validate = false;
                    break;
                }
            }
            return this.custom((/**
             * @return {?}
             */
            () => validate), validators, controls)(control);
        });
    }
    /**
     * Clean subscriptions for changes in control value
     * @private
     * @param {?} control controls array
     * @return {?}
     */
    static cleanChangeSubscriptions(control) {
        if (!(control.changeSubscriptions instanceof Array)) {
            control.changeSubscriptions = [];
        }
        for (const subscription of control.changeSubscriptions) {
            subscription.unsubscribe();
        }
        control.changeSubscriptions = [];
    }
    /**
     * Subscribe to changes in the values of the controls to validate the main control
     * @private
     * @param {?} control main control
     * @param {?} controls controls array
     * @return {?}
     */
    static pushChangeSubscriptions(control, controls) {
        if (!(control.changeSubscriptions instanceof Array)) {
            control.changeSubscriptions = [];
        }
        controls = this.toRealControls(control, controls);
        for (const conditionControl of controls) {
            /** @type {?} */
            const realControl = typeof conditionControl === 'string' ?
                this.getControlByName(conditionControl, (/** @type {?} */ (control.parent))) :
                conditionControl;
            if (realControl === null) {
                throw new Error(`The "${conditionControl}" control does not exist.`);
            }
            /** @type {?} */
            const subscription = realControl.valueChanges.subscribe((/**
             * @param {?} e
             * @return {?}
             */
            e => control.updateValueAndValidity()));
            control.changeSubscriptions.push(subscription);
        }
    }
    /**
     * Get the control object from group parent
     * @private
     * @param {?} name control name to get
     * @param {?} group group parent
     * @return {?}
     */
    static getControlByName(name, group) {
        for (const key in group.controls) {
            if (key === name) {
                return group.controls[key];
            }
        }
        return null;
    }
    /**
     * Get abstract control array
     * @private
     * @param {?} control main control
     * @param {?} controls abstract controls or abstract controls name
     * @return {?}
     */
    static toRealControls(control, controls) {
        /** @type {?} */
        const data = [];
        if (!(controls instanceof Array)) {
            controls = [];
        }
        for (const conditionControl of controls) {
            /** @type {?} */
            const realControl = typeof conditionControl === 'string' ?
                this.getControlByName(conditionControl, (/** @type {?} */ (control.parent))) :
                conditionControl;
            if (realControl === null) {
                throw new Error(`The "${conditionControl}" control does not exist.`);
            }
            data.push(realControl);
        }
        return data;
    }
}

export { ConditionalValidators };
//# sourceMappingURL=ngx-conditional-validation.js.map
